#include "FreeRTOSConfig.h"
#include "nios2.h"


# Allows assembly code to use at register without warning. Macro or relaxation expansions generate warnings.
.set noat

# Exported to start the first task.
.globl restore_sp_from_pxCurrentTCB
.globl isr_handler

.section .exceptions

isr_handler:
	addi	ea, ea, -4			# Point to the next instruction.								
	addi	sp,	sp, -124		# Create space on the stack.

	# store registers to stack
	stw		ra, 8(sp)
								# Leave a gap for muldiv 0
	stw		at, 16(sp)

	# Debug signal (LED6)
	#movhi 	et,     %hi(LED_BASE)	# load the address of LED controller
	#ori 	et, et, %lo(LED_BASE)
	#movi	at, 64
	#stwio	at, 16(et)			# write the PIO set register (offset 4*4=16)

	# disable irqs with CPU IL field (interrupt level)
	rdctl   et, status			# read status
	movhi 	at,     %hi(~NIOS2_STATUS_IL_MSK)	# load inverter status il mask
	ori 	at, at, %lo(~NIOS2_STATUS_IL_MSK)
	and     et, et, at 			# set il field in status to 0
	ori     et, et, (configMAX_SYSCALL_INTERRUPT_PRIORITY << NIOS2_STATUS_IL_OFST) # set il field in status to configMAX_SYSCALL_INTERRUPT_PRIORITY

	# enable PIE		
	ori     et, et, NIOS2_STATUS_PIE_MSK	# enable PIE because all interrupts run with shadow registers
	wrctl	status, et 			# write status

	# continue to store registers
	stw		r2, 20(sp)
	stw		r3, 24(sp)
	stw		r4, 28(sp)
	stw		r5, 32(sp)
	stw		r6, 36(sp)
	stw		r7, 40(sp)
	stw		r8, 44(sp)
	stw		r9, 48(sp)
	stw		r10, 52(sp)
	stw		r11, 56(sp)
	stw		r12, 60(sp)
	stw		r13, 64(sp)
	stw		r14, 68(sp)
	stw		r15, 72(sp)
	rdctl	r5, estatus 		# Save the eStatus
	stw		r5, 76(sp)
	stw		ea, 80(sp)			# Save the PC
	stw		r16, 84(sp)			# Save the remaining registers
	stw		r17, 88(sp)
	stw		r18, 92(sp)
	stw		r19, 96(sp)
	stw		r20, 100(sp)
	stw		r21, 104(sp)
	stw		r22, 108(sp)
	stw		r23, 112(sp)
	stw		gp, 116(sp)
	stw		fp, 120(sp)

save_sp_to_pxCurrentTCB:
	movia	et, pxCurrentTCB	# Load the address of the pxCurrentTCB pointer
	ldw		et, (et)			# Load the value of the pxCurrentTCB pointer
	stw		sp, (et)			# Store the stack pointer into the top of the TCB

	call 	vPortSysTickHandler()	# call tick handler

restore_sp_from_pxCurrentTCB:
	movia	et, pxCurrentTCB		# Load the address of the pxCurrentTCB pointer
	ldw		et, (et)				# Load the value of the pxCurrentTCB pointer
	ldw		sp, (et)				# Load the stack pointer with the top value of the TCB

restore_context:

	ldw		ra, 8(sp)		# Restore the registers.
							# Leave a gap for muldiv 0.
	#at will be restored at the end!
	ldw		r2, 20(sp)
	ldw		r3, 24(sp)
	ldw		r4, 28(sp)
	ldw		r5, 32(sp)
	ldw		r6, 36(sp)
	ldw		r7, 40(sp)
	ldw		r8, 44(sp)
	ldw		r9, 48(sp)
	ldw		r10, 52(sp)
	ldw		r11, 56(sp)
	ldw		r12, 60(sp)
	ldw		r13, 64(sp)
	ldw		r14, 68(sp)
	ldw		r15, 72(sp)
	ldw		et, 76(sp)		# Load the eStatus
	wrctl	estatus, et 	# Write the eStatus
	ldw		ea, 80(sp)		# Load the Program Counter
	ldw		r16, 84(sp)
	ldw		r17, 88(sp)
	ldw		r18, 92(sp)
	ldw		r19, 96(sp)
	ldw		r20, 100(sp)
	ldw		r21, 104(sp)
	ldw		r22, 108(sp)
	ldw		r23, 112(sp)
	ldw		gp, 116(sp)
	ldw		fp, 120(sp)

	# disable PIE
	rdctl   et, status			# read status
	movhi 	at,     %hi(~NIOS2_STATUS_PIE_MSK)	# load inverter status pie mask
	ori 	at, at, %lo(~NIOS2_STATUS_PIE_MSK)
	and		et, et, at			# clear PIE bit
	wrctl	status, et 			# write status

	# Debug signal (LED6)
	#movhi 	et,     %hi(LED_BASE)	# load the address of LED controller
	#ori 	et, et, %lo(LED_BASE)
	#movi	at, 64
	#stwio	at, 20(et)			# write the PIO clear register (offset 5*4=20)

	ldw		at, 16(sp)		# restore at

	addi	sp,	sp, 124		# Release stack space
    eret					# Return to address ea, loading eStatus into Status.

#---------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------
# code for exception handling, specially for the trap instruction (will be executed
# because of portYIELD(), see portmacro.h
# almost a copy of normal interrupt code, slightly diffrent because of shadow registers
#---------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------

.section .exceptions.entry.label, "xa"
.globl exception_handler
.type exception_handler, @function

exception_handler:
	# exception handling...
	rdctl et, exception
	srli et, et, 2
	addi et, et, -3 		# 3 is the code for the trap instruction
	beq et, zero, handle_trap  # its a trap instruction...

unhandled_exception:
	addi et, et, 3		# add 3 to restore the exeption cause in et
	break
	br unhandled_exception

handle_trap:
# save context, almost the same as in isr.
# ea already points to the next instruction, dont decrement it by 4!
# call vTaskSwitchContext insted of vPortSysTickHandler

# save_context:
	addi	sp,	sp, -124		# Create space on the stack.

	# store registers to stack
	stw		ra, 8(sp)
								# Leave a gap for muldiv 0
	stw		at, 16(sp)

	# Debug signal (LED1)
	#movhi 	et,     %hi(LED_BASE)	# load the address of LED controller
	#ori 	et, et, %lo(LED_BASE)
	#movi	at, 2
	#stwio	at, 16(et)			# write the PIO set register (offset 4*4=16)

	# disable irqs with CPU IL field (interrupt level)
	rdctl   et, status			# read status
	movhi 	at,     %hi(~NIOS2_STATUS_IL_MSK)	# load inverter status il mask
	ori 	at, at, %lo(~NIOS2_STATUS_IL_MSK)
	and     et, et, at 			# set il field in status to 0
	ori     et, et, (configMAX_SYSCALL_INTERRUPT_PRIORITY << NIOS2_STATUS_IL_OFST) # set il field in status to configMAX_SYSCALL_INTERRUPT_PRIORITY

	# enable PIE		
	ori     et, et, NIOS2_STATUS_PIE_MSK	# enable PIE because all interrupts run with shadow registers
	wrctl	status, et 			# write status

	# continue to store registers
	stw		r2, 20(sp)
	stw		r3, 24(sp)
	stw		r4, 28(sp)
	stw		r5, 32(sp)
	stw		r6, 36(sp)
	stw		r7, 40(sp)
	stw		r8, 44(sp)
	stw		r9, 48(sp)
	stw		r10, 52(sp)
	stw		r11, 56(sp)
	stw		r12, 60(sp)
	stw		r13, 64(sp)
	stw		r14, 68(sp)
	stw		r15, 72(sp)
	rdctl	r5, estatus 		# Save the eStatus
	stw		r5, 76(sp)
	stw		ea, 80(sp)			# Save the PC
	stw		r16, 84(sp)			# Save the remaining registers
	stw		r17, 88(sp)
	stw		r18, 92(sp)
	stw		r19, 96(sp)
	stw		r20, 100(sp)
	stw		r21, 104(sp)
	stw		r22, 108(sp)
	stw		r23, 112(sp)
	stw		gp, 116(sp)
	stw		fp, 120(sp)


# save_sp_to_pxCurrentTCB
	movia	et, pxCurrentTCB	# Load the address of the pxCurrentTCB pointer
	ldw		et, (et)			# Load the value of the pxCurrentTCB pointer
	stw		sp, (et)			# Store the stack pointer into the top of the TCB

# call_scheduler:
	call	vTaskSwitchContext				# Pick the next context.

#restore_sp_from_pxCurrentTCB:
	movia	et, pxCurrentTCB		# Load the address of the pxCurrentTCB pointer
	ldw		et, (et)				# Load the value of the pxCurrentTCB pointer
	ldw		sp, (et)				# Load the stack pointer with the top value of the TCB

# restore_context
	ldw		ra, 8(sp)		# Restore the registers.
							# Leave a gap for muldiv 0.
	#at will be restored at the end!
	ldw		r2, 20(sp)
	ldw		r3, 24(sp)
	ldw		r4, 28(sp)
	ldw		r5, 32(sp)
	ldw		r6, 36(sp)
	ldw		r7, 40(sp)
	ldw		r8, 44(sp)
	ldw		r9, 48(sp)
	ldw		r10, 52(sp)
	ldw		r11, 56(sp)
	ldw		r12, 60(sp)
	ldw		r13, 64(sp)
	ldw		r14, 68(sp)
	ldw		r15, 72(sp)
	ldw		et, 76(sp)		# Load the eStatus
	wrctl	estatus, et 	# Write the eStatus
	ldw		ea, 80(sp)		# Load the Program Counter
	ldw		r16, 84(sp)
	ldw		r17, 88(sp)
	ldw		r18, 92(sp)
	ldw		r19, 96(sp)
	ldw		r20, 100(sp)
	ldw		r21, 104(sp)
	ldw		r22, 108(sp)
	ldw		r23, 112(sp)
	ldw		gp, 116(sp)
	ldw		fp, 120(sp)

	# disable PIE
	rdctl   et, status			# read status
	movhi 	at,     %hi(~NIOS2_STATUS_PIE_MSK)	# load inverter status pie mask
	ori 	at, at, %lo(~NIOS2_STATUS_PIE_MSK)
	and		et, et, at			# clear PIE bit
	wrctl	status, et 			# write status

	# Debug signal LED1
	#movhi 	et,     %hi(LED_BASE)	# load the address of LED controller
	#ori 	et, et, %lo(LED_BASE)
	#movi	at, 2
	#stwio	at, 20(et)			# write the PIO clear register (offset 5*4=20)

	ldw		at, 16(sp)		# restore at

	addi	sp,	sp, 124		# Release stack space
    eret					# Return to address ea, loading eStatus into Status.


