/*
 * FreeRTOS Kernel V10.2.0
 * Copyright (C) 2019 Amazon.com, Inc. or its affiliates.  All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * http://www.FreeRTOS.org
 * http://aws.amazon.com/freertos
 *
 * 1 tab == 4 spaces!
 */

#ifndef PORTMACRO_H
#define PORTMACRO_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include "altera_vic_irq.h"

extern void vTaskEnterCritical( void );
extern void vTaskExitCritical( void );

void vAssertCalled(const char* file, uint32_t line);
void configRunTimeStats(void);
uint32_t getRunTimeStatsCounterValue(void);
void traceTaskSwitchedIn(void);
void traceTaskSwitchedOut(void);

/* Architecture specifics. */
#define portSTACK_GROWTH				( -1 )
#define portBYTE_ALIGNMENT				4
#define portCRITICAL_NESTING_IN_TCB		1
#define portNOP()                   	asm volatile ( "NOP" )
#define portTICK_PERIOD_MS				( ( TickType_t ) 1000 / configTICK_RATE_HZ )
#define portBASE_TYPE					long

/* Type definitions. */
#define portSTACK_TYPE					uint32_t
typedef portSTACK_TYPE 					StackType_t;
typedef long 							BaseType_t;
typedef unsigned long 					UBaseType_t;
#if( configUSE_16_BIT_TICKS == 1 )
	typedef uint16_t 					TickType_t;
	#define portMAX_DELAY ( TickType_t ) 0xffff
#else
	typedef uint32_t 					TickType_t;
	#define portMAX_DELAY ( TickType_t ) 0xffffffffUL

	/* 32-bit tick type on a 32-bit architecture, so reads of the tick count do
	not need to be guarded with a critical section. */
	#define portTICK_TYPE_IS_ATOMIC 1
#endif

/* ported functions */
void vPortDisableInterrupts( void ); // implemented in port.c
void vPortEnableInterrupts( void ); // implemented in port.c
#define portYIELD()						asm volatile ( "trap" );

// fire software interrupt to RTOS SysTick Timer
//#define portYIELD_FROM_ISR( x ) 		if( ( x ) ) IOWR_ALTERA_VIC_SW_INTERRUPT_SET(alt_vic_dev_list[0]->base, 1)
#define portYIELD_FROM_ISR( x ) 		\
					do{ \
						if( ( x ) ){ \
							IOWR_ALTERA_VIC_SW_INTERRUPT_SET(alt_vic_dev_list[0]->base, 1); \
						} \
					}while(0)


#define portDISABLE_INTERRUPTS()		vPortDisableInterrupts()
#define portENABLE_INTERRUPTS()			vPortEnableInterrupts()
#define portENTER_CRITICAL()        	vTaskEnterCritical()
#define portEXIT_CRITICAL()         	vTaskExitCritical()
#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) void vFunction( void *pvParameters )
#define portTASK_FUNCTION( vFunction, pvParameters ) void vFunction( void *pvParameters )

UBaseType_t vPortSetInterruptMaskFromISR(void);
void vPortClearInterruptMaskFromISR(UBaseType_t uxSavedInterruptStatus);

#define portSET_INTERRUPT_MASK_FROM_ISR()		vPortSetInterruptMaskFromISR()
#define portCLEAR_INTERRUPT_MASK_FROM_ISR(uxSavedInterruptStatus)		vPortClearInterruptMaskFromISR(uxSavedInterruptStatus)

#ifdef __cplusplus
}
#endif

#endif /* PORTMACRO_H */

